{
	//- Momentum equation
	coupledFvVectorMatrix UEqns(2);

	UEqns.setEqn
	(
	    0,
	    new fvVectorMatrix
	    (
	    	fvm::ddt(U)
          + fvm::div(phi, U)
          + turb->divDevReff(U)
          == 
          - fvc::grad(p)
	    )
	);

	UEqns.setEqn
	(
	    1,
	    new fvVectorMatrix
	    (
	    	fvm::ddt(UTwo)
          + fvm::div(phiTwo, UTwo)
          + turb->divDevReff(UTwo)
          == 
          - fvc::grad(pTwo)
	    )
	);

	UEqns.solve(mesh.solver("coupled_U"));
}

for (int corr=0; corr<nCorr; corr++)
{
    //- Pressure equations
    coupledFvScalarMatrix pEqns(2);

    fvVectorMatrix UEqn
    (
        fvm::ddt(U)
      + fvm::div(phi, U)
      + turb->divDevReff(U)
    );

    volScalarField rAU(1.0/UEqn.A());

    volVectorField HbyA("HbyA", U);
    HbyA = rAU*UEqn.H();
    surfaceScalarField phiHbyA
    (
        "phiHbyA",
        (fvc::interpolate(HbyA) & mesh.Sf())
      +  fvc::interpolate(rAU)*fvc::ddtCorr(U, phi)
    );

    //adjustPhi(phiHbyA, U, p);

    fvVectorMatrix UEqnTwo
    (
        fvm::ddt(UTwo)
      + fvm::div(phiTwo, UTwo)
      + turb->divDevReff(UTwo)
    );

    volScalarField rAUTwo(1.0/UEqnTwo.A());

    volVectorField HbyATwo("HbyA", UTwo);
    HbyATwo = rAUTwo*UEqnTwo.H();
    surfaceScalarField phiHbyATwo
    (
        "phiHbyA",
        (fvc::interpolate(HbyATwo) & meshTwo.Sf())
      + fvc::interpolate(rAUTwo)*fvc::ddtCorr(UTwo, phiTwo)
    );

    //adjustPhi(phiHbyATwo, UTwo, pTwo);
    
    // Non-orthogonal pressure corrector loop
    for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
    {
        // Pressure corrector

        fvScalarMatrix pEqn
        (
            fvm::laplacian(rAU, p) == fvc::div(phiHbyA)
        );

        fvScalarMatrix pEqnTwo
        (
            fvm::laplacian(rAUTwo, pTwo) == fvc::div(phiHbyATwo)
        );

        pEqns.setEqn
        (
            0,
            new fvScalarMatrix
            (
                pEqn
            )
        );


        pEqns.setEqn
        (
            1,
            new fvScalarMatrix
            (
                pEqnTwo
            )
        );

        pEqns.solve();

        if (nonOrth == nNonOrthCorr)
        {
            phi = phiHbyA - pEqn.flux();
            phiTwo = phiHbyATwo - pEqnTwo.flux();
        }
    }
     #include "continuityErrs.H"

    /*if (corr == nCorr - 1)
    {*/
        U = fvc::reconstruct(phi);
        UTwo = fvc::reconstruct(phiTwo);
    /*}
    else
    {
        U = HbyA - rAU*fvc::grad(p);
        UTwo = HbyATwo - rAUTwo*fvc::grad(pTwo);
    }*/

    U.correctBoundaryConditions(); 
    UTwo.correctBoundaryConditions(); 
}