/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    coupledBicgStabSolver

Description
    Preconditioned Stabilised Bi-Conjugate Gradient solver

Author
    Hrvoje Jasak, Wikki Ltd.  All rights reserved

Rewriting for OpenFOAM 2.3.0
    Bahram Haddadi, TUWards OpenFOAM.  All rights reserved

SourceFiles
    coupledBicgStabSolver.C

\*---------------------------------------------------------------------------*/

#ifndef coupledBicgStabSolver_H
#define coupledBicgStabSolver_H

#include "coupledIterativeSolver.H"
#include "coupledLduPrecon.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                      Class coupledBicgStabSolver Declaration
\*---------------------------------------------------------------------------*/

class coupledBicgStabSolver
:
    public coupledIterativeSolver
{
    // Private Data

        //- Type definition
        typedef List<lduInterfaceFieldPtrsList> lduInterfaceFieldPtrsListList;

        // Gauss-Seidel preconditioner
        autoPtr<coupledLduPrecon> preconPtr_;


    // Private Member Functions

        //- Disallow default bitwise copy construct
        coupledBicgStabSolver(const coupledBicgStabSolver&);

        //- Disallow default bitwise assignment
        void operator=(const coupledBicgStabSolver&);


public:

    //- Runtime type information
    TypeName("BiCGStab");


    // Constructors

        //- Construct from matrix
        coupledBicgStabSolver
        (
            const word& fieldName,
            const coupledLduMatrix& matrix,
            const PtrList<FieldField<Field, scalar> >& bouCoeffs,
            const PtrList<FieldField<Field, scalar> >& intCoeffs,
            const lduInterfaceFieldPtrsListList& interfaces,
            const dictionary& solverData
        );


    // Destructor

        virtual ~coupledBicgStabSolver()
        {}


    // Member Functions

        //- Solve the matrix with this solver
        virtual coupledSolverPerformance solve
        (
            FieldField<Field, scalar>& x,
            const FieldField<Field, scalar>& b,
            const direction cmpt = 0
        ) const;


        template<class Type>
        scalar gSumProd
        (
            const FieldField<Field, Type>& f1,
            const FieldField<Field, Type>& f2
        ) const
        {
            scalar SumProd = 0;

            if (f1.size() && (f1.size() == f2.size()))
            {
                forAll (f1, i)
                {
                    SumProd += sumProd(f1[i], f2[i]);
                }
                reduce(SumProd, sumOp<scalar>());
            }

            return SumProd;
        }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
